const _ = require('lodash');
const MAX_SIZE = 10;
const MAX_DEEP = 10;
/**
 * - if is array and is not from collection, reduce the collection to MAX_SIZE;
 * - if is object and is not from collection, call recursively processtoLog fn;
 * - if is array and is from collection, returns [...]
 * - if is object and is from collection, returns {...}
 * - else, returns the same value.
 */
const processToLog = (data, fromCollection = false, isCollection = false, deep = 0) => {
    const isArray = _.isArray(data);
    const isObject = _.isPlainObject(data);
    deep++;

    // protects from infinity recursion. This will interrupt if deep is greater than maxDeep
    if (deep > MAX_DEEP) {
        return '{...}'
    }

    if (isArray && !fromCollection) {
        const total = data.length;
        const result = _.chain(data).take(MAX_SIZE).map(i => processToLog(i, fromCollection, true, deep)).value();
        if (total > MAX_SIZE) {
            result.push(`[${total - MAX_SIZE} more results]`)
        }

        return result;
    } else if (isObject && !fromCollection) {
        return Object.keys(data).reduce((obj, key) => {
            obj[key] = processToLog(data[key], isCollection, isCollection, deep);
            return obj;
        }, {});
    } else if (isArray) {
        return '[...]';
    } else if (isObject) {
        return  '{...}';
    } else {
        return data;
    }
};

const defaultResponseProcessor = data => processToLog(data);

module.exports = defaultResponseProcessor;