const contractorRepository = require('../repositories/contractor.repository');
const logRepository = require('../repositories/log.repository');
const resultRepository = require('../repositories/result.repository');
const typeRepository = require('../repositories/type.repository');
const os = require('os');
class Logger {
    constructor () {
        this.meta = {};
    }

    setRequestMetaData (meta) {
        this.meta = meta;
    }

    qualifyUser (user) {
        this.meta.user = user;
    }

    log (type, reason, data, result, system, contractorId = this.meta.contractorId, date = new Date()) {
        if (!contractorId) {
            return false;
        }

        system = { _id: parseInt(system) };
        type = typeRepository.getByDescription(contractorId, type);
        result = resultRepository.getByDescription(contractorId, result);
        const serviceName = reason.match(/^\w+/)[0];
        const log = {
            ip: this.meta.clientIP,
            port: this.meta.clientPort,
            user: this.meta.user,
            request_id: this.meta.request_id,
            date,
            data,
            reason,
            result,
            type,
            system,
            service: {
                name: serviceName,
                ip: this.meta.serverIP,
                host: os.hostname()
            }
        };

        return logRepository.save(contractorId, log);
    }

    broadcast (type, reason, data, result, system) {
        const date = new Date();
        const promise = contractorRepository.list().then(contractors => contractors.map(contractor => {
            return contractor.active && this.log(type, reason, data, result, system, contractor._id, date);
        }));

        return promise;
    };

}

module.exports = Logger;
